#!/usr/bin/env python3

import functools
import json
import stanza
import sys
import torch
from tqdm import tqdm

def init_processor(lang):
    if lang == 'cs':
        nlp = stanza.Pipeline(lang='cs', package='pdt', processors='tokenize,mwt,pos,lemma,depparse')
    else:
        nlp = stanza.Pipeline(lang=lang, processors='tokenize,mwt,pos,lemma,depparse')
    return nlp


def serialize_context(data):
    words = dict()
    total = 0
    current = 0
    for i in range(len(data.sentences)):
        for w in data.sentences[i].words:
            current += 1
            words[w.id + total] = {'word': w.text, 'lemma': w.lemma, 'tags': w.xpos, 'class': w.upos}
        total += current
        current = 0
    return words


def serialize_entity(data, words):
    etok = []
    for i in range(len(data.sentences)):
        for w in data.sentences[i].words:
            etok.append(w.text)
    
    tokens = [words[x]['word'] for x in sorted(words)]
    start = 0
    for i in range(len(tokens)):
        if tokens[i : i + len(etok)] == etok:
            start = i
            break
    end = start + len(etok)

    tree = dict()
    for i in range(len(data.sentences)):
        for w in data.sentences[i].words:
            if (w.head + start) in tree:
                tree[w.head + start].append(w.id + start)
            else:
                tree[w.head + start] = [w.id + start]
    return tree, start, end


def analyse_file(filename, out_filename):
    data = None
    with open(filename, 'r', encoding='UTF-8') as rf:
        data = json.load(rf)
    id_split = dict()
    for i in data:
        lang = data[i]['lang']
        if lang in id_split:
            id_split[lang].append(i)
        else:
            id_split[lang] = [i]
    found_langs = set(id_split.keys())
    
    analysed_data = dict()
    for lang in tqdm(list(found_langs)):
        processor = init_processor(lang)
        for i in tqdm(id_split[lang]):
            cdict = data[i]
            cdata = processor(cdict['context'])
            edata = list()
            for e in cdict['entities']:
                edata.append(processor(e))
            analysed_data[int(i)] = {'cdata': cdata, 'edata': edata, 'lang': lang, 'original': cdict['entities'], 'types': None if 'types' not in cdict else cdict['types'], 'ids': None if 'ids' not in cdict else cdict['ids']}
    out_data = list()
    for datapoint in sorted(analysed_data):
        words = serialize_context(analysed_data[datapoint]['cdata'])
        trees = list()
        for e in analysed_data[datapoint]['edata']:
            tree, start, end = serialize_entity(e, words)
            trees.append((tree, start, end))
        out_data.append({'words': words, 'trees': trees, 'lang': analysed_data[datapoint]['lang'], 'original': analysed_data[datapoint]['original'], 'types': analysed_data[datapoint]['types'], 'ids': analysed_data[datapoint]['ids']}) 
        
    
    with open(out_filename, 'w', encoding='UTF-8') as wf:
        json.dump(out_data, wf)


analyse_file(sys.argv[1], sys.argv[2])
